'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withBorder = exports.border = undefined;

var _merge = require('ramda/src/merge');

var _merge2 = _interopRequireDefault(_merge);

var _is = require('ramda/src/is');

var _is2 = _interopRequireDefault(_is);

var _filter = require('ramda/src/filter');

var _filter2 = _interopRequireDefault(_filter);

var _toPairs = require('ramda/src/toPairs');

var _toPairs2 = _interopRequireDefault(_toPairs);

var _identity = require('ramda/src/identity');

var _identity2 = _interopRequireDefault(_identity);

var _ifElse = require('ramda/src/ifElse');

var _ifElse2 = _interopRequireDefault(_ifElse);

var _tap = require('ramda/src/tap');

var _tap2 = _interopRequireDefault(_tap);

var _head = require('ramda/src/head');

var _head2 = _interopRequireDefault(_head);

var _defaultTo = require('ramda/src/defaultTo');

var _defaultTo2 = _interopRequireDefault(_defaultTo);

var _length = require('ramda/src/length');

var _length2 = _interopRequireDefault(_length);

var _ = require('ramda/src/__');

var _2 = _interopRequireDefault(_);

var _gt = require('ramda/src/gt');

var _gt2 = _interopRequireDefault(_gt);

var _compose = require('ramda/src/compose');

var _compose2 = _interopRequireDefault(_compose);

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _styleHelper = require('./style-helper');

var _utils = require('./utils');

var _mediaQueries = require('./media-queries');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/**
 * Scale
 */
var border = exports.border = {
  radii: [0, 1, 2, 3, 4, 100, 'pill'],
  styles: ['none', 'dotted', 'dashed', 'solid'],
  widths: [0, 1, 2, 3, 4, 5]
};

/**
 * Normalize classnames to add a `-`
 * prefix when needed
 */
var normalizeRadii = (0, _utils.normalizeClassNames)([100, 'pill']);

/**
 * Parse border declaration to get
 * border type and color values
 */
var hasMultipleDeclarations = (0, _compose2.default)((0, _gt2.default)(_2.default, 1), _length2.default);
var headOr = function headOr(def) {
  return (0, _compose2.default)((0, _defaultTo2.default)(def), _head2.default);
};
var warnAboutMultipleDeclarations = (0, _tap2.default)(function () {
  console.warn('Warning: multiple border types declared.\n    You can only declare one border type (ba, bl, br, bt, bb, bn) at a time.');
});

var borderDeclaration = (0, _compose2.default)(headOr([]), (0, _ifElse2.default)(hasMultipleDeclarations, warnAboutMultipleDeclarations, _identity2.default), _toPairs2.default, (0, _filter2.default)(_utils.isPresent));

/**
 * withBorder HOC
 */
function borderTransform(_ref) {
  var className = _ref.className,
      ba = _ref.ba,
      bl = _ref.bl,
      br = _ref.br,
      bt = _ref.bt,
      bb = _ref.bb,
      bn = _ref.bn,
      bw = _ref.bw,
      radius = _ref.radius,
      rounded = _ref.rounded,
      ownerProps = _objectWithoutProperties(_ref, ['className', 'ba', 'bl', 'br', 'bt', 'bb', 'bn', 'bw', 'radius', 'rounded']);

  var _borderDeclaration = borderDeclaration({ ba: ba, bl: bl, br: br, bt: bt, bb: bb, bn: bn }),
      _borderDeclaration2 = _slicedToArray(_borderDeclaration, 2),
      type = _borderDeclaration2[0],
      value = _borderDeclaration2[1];

  var borderColor = (0, _is2.default)(Boolean, value) ? undefined : value;

  var base = {
    br: normalizeRadii(radius),
    'br--': rounded,
    'b--': borderColor,
    bw: bw
  };

  var classList = type ? (0, _merge2.default)(base, _defineProperty({}, type, '')) : base;

  var borderStyles = (0, _styleHelper.classesFor)(classList);

  return (0, _merge2.default)({ className: (0, _utils.cx)([borderStyles, className]) }, ownerProps);
}

var borderType = function borderType(colors) {
  return _propTypes2.default.oneOfType([_propTypes2.default.oneOf(colors), _propTypes2.default.bool]);
};

var borderPropTypes = function borderPropTypes(colors) {
  return {
    ba: borderType(colors),
    bl: borderType(colors),
    br: borderType(colors),
    bt: borderType(colors),
    bb: borderType(colors),
    bn: _propTypes2.default.bool,
    bw: (0, _mediaQueries.addMQSupport)(_propTypes2.default.oneOf(border.widths)),
    radius: (0, _mediaQueries.addMQSupport)(_propTypes2.default.oneOf(border.radii)),
    rounded: (0, _mediaQueries.addMQSupport)(_propTypes2.default.oneOf(['bottom', 'top', 'right', 'left'])),
    className: _propTypes2.default.any
  };
};

var withBorder = exports.withBorder = function withBorder(colors) {
  return function (component) {
    return (0, _utils.createWithStyleHoc)({
      name: 'withBorder',
      transformation: borderTransform,
      propTypes: borderPropTypes(colors),
      component: component
    });
  };
};