'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withSize = exports.heights = exports.widths = undefined;

var _merge = require('ramda/src/merge');

var _merge2 = _interopRequireDefault(_merge);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _styleHelper = require('./style-helper');

var _utils = require('./utils');

var _mediaQueries = require('./media-queries');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/**
 * Scale
 */
var widths = exports.widths = [1, 2, 3, 4, 5, 10, 20, 25, 30, 33, 34, 40, 50, 60, 70, 75, 80, 90, 100, 'third', 'two-thirds', 'auto'];

var heights = exports.heights = [1, 2, 3, 4, 5, 25, 50, 75, 100, 'auto'];

/**
 * Normalize classnames to add a `-`
 * prefix when needed
 */
var normalizeWidths = (0, _utils.normalizeClassNames)([10, 20, 25, 30, 33, 34, 40, 50, 60, 70, 75, 80, 90, 100, 'third', 'two-thirds', 'auto']);

var normalizeHeights = (0, _utils.normalizeClassNames)([25, 50, 75, 100, 'auto']);

/**
 * withSize HOC
 */
function sizeTransform(_ref) {
  var className = _ref.className,
      w = _ref.w,
      h = _ref.h,
      ownerProps = _objectWithoutProperties(_ref, ['className', 'w', 'h']);

  var sizes = (0, _styleHelper.classesFor)({ w: normalizeWidths(w), h: normalizeHeights(h) });

  return (0, _merge2.default)({ className: (0, _utils.cx)([sizes, className]) }, ownerProps);
}

var sizePropTypes = {
  w: (0, _mediaQueries.addMQSupport)(_propTypes2.default.oneOf(widths)),
  h: (0, _mediaQueries.addMQSupport)(_propTypes2.default.oneOf(heights)),
  className: _propTypes2.default.any
};

var withSize = exports.withSize = function withSize(component) {
  return (0, _utils.createWithStyleHoc)({
    name: 'withSize',
    transformation: sizeTransform,
    propTypes: sizePropTypes,
    component: component
  });
};