'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withSpacing = exports.negativeMarginScale = exports.whiteSpaceScale = undefined;

var _merge = require('ramda/src/merge');

var _merge2 = _interopRequireDefault(_merge);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _styleHelper = require('./style-helper');

var _utils = require('./utils');

var _mediaQueries = require('./media-queries');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/**
 * Scale
 */
var whiteSpaceScale = exports.whiteSpaceScale = [0, 1, 2, 3, 4, 5, 6, 7];
var negativeMarginScale = exports.negativeMarginScale = [1, 2, 3, 4, 5, 6, 7];

/**
 * withSpacing HOC
 */
function spacingTransform(_ref) {
  var className = _ref.className,
      ma = _ref.ma,
      mt = _ref.mt,
      ml = _ref.ml,
      mr = _ref.mr,
      mb = _ref.mb,
      mv = _ref.mv,
      mh = _ref.mh,
      pa = _ref.pa,
      pt = _ref.pt,
      pl = _ref.pl,
      pr = _ref.pr,
      pb = _ref.pb,
      pv = _ref.pv,
      ph = _ref.ph,
      na = _ref.na,
      nt = _ref.nt,
      nl = _ref.nl,
      nr = _ref.nr,
      nb = _ref.nb,
      ownerProps = _objectWithoutProperties(_ref, ['className', 'ma', 'mt', 'ml', 'mr', 'mb', 'mv', 'mh', 'pa', 'pt', 'pl', 'pr', 'pb', 'pv', 'ph', 'na', 'nt', 'nl', 'nr', 'nb']);

  var margins = (0, _styleHelper.classesFor)({ ma: ma, mt: mt, ml: ml, mr: mr, mb: mb, mv: mv, mh: mh });
  var negativeMargins = (0, _styleHelper.classesFor)({ na: na, nt: nt, nl: nl, nr: nr, nb: nb });
  var paddings = (0, _styleHelper.classesFor)({ pa: pa, pt: pt, pl: pl, pr: pr, pb: pb, pv: pv, ph: ph });

  return (0, _merge2.default)({ className: (0, _utils.cx)([margins, negativeMargins, paddings, className]) }, ownerProps);
}

var spacingProp = (0, _mediaQueries.addMQSupport)(_propTypes2.default.oneOf(whiteSpaceScale));
var negativeMarginProp = (0, _mediaQueries.addMQSupport)(_propTypes2.default.oneOf(negativeMarginScale));

var spacingPropTypes = {
  ma: spacingProp,
  mt: spacingProp,
  ml: spacingProp,
  mr: spacingProp,
  mb: spacingProp,
  mv: spacingProp,
  mh: spacingProp,
  na: negativeMarginProp,
  nt: negativeMarginProp,
  nl: negativeMarginProp,
  nr: negativeMarginProp,
  nb: negativeMarginProp,
  pa: spacingProp,
  pt: spacingProp,
  pl: spacingProp,
  pr: spacingProp,
  pb: spacingProp,
  pv: spacingProp,
  ph: spacingProp,
  className: _propTypes2.default.any
};

var withSpacing = exports.withSpacing = function withSpacing(component) {
  return (0, _utils.createWithStyleHoc)({
    name: 'withSpacing',
    transformation: spacingTransform,
    propTypes: spacingPropTypes,
    component: component
  });
};