'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withTypography = exports.typeScale = exports.lineHeights = undefined;

var _merge = require('ramda/src/merge');

var _merge2 = _interopRequireDefault(_merge);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _styleHelper = require('./style-helper');

var _utils = require('./utils');

var _mediaQueries = require('./media-queries');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/**
 * Scale
 */
var lineHeights = exports.lineHeights = ['solid', 'title', 'copy'];
var typeScale = exports.typeScale = [1, 2, 3, 4, 5, 6, 7, 'headline', 'subheadline'];

/**
 * Normalize classnames to add a `-`
 * prefix when needed
 */
var normalizeFontSizes = (0, _utils.normalizeClassNames)(['headline', 'subheadline']);

/**
 * withTypography HOC
 */
function typographyTransform(_ref) {
  var className = _ref.className,
      f = _ref.f,
      lh = _ref.lh,
      ownerProps = _objectWithoutProperties(_ref, ['className', 'f', 'lh']);

  var type = (0, _styleHelper.classesFor)({ f: normalizeFontSizes(f), 'lh-': lh });

  return (0, _merge2.default)({ className: (0, _utils.cx)([type, className]) }, ownerProps);
}

var sizePropTypes = {
  f: (0, _mediaQueries.addMQSupport)(_propTypes2.default.oneOf(typeScale)),
  lh: (0, _mediaQueries.addMQSupport)(_propTypes2.default.oneOf(lineHeights)),
  className: _propTypes2.default.any
};

var withTypography = exports.withTypography = function withTypography(component) {
  return (0, _utils.createWithStyleHoc)({
    name: 'withTypography',
    transformation: typographyTransform,
    propTypes: sizePropTypes,
    component: component
  });
};